@@ -1,11 +1,8 @@
 #### Symmetric Sparse Matrices in compressed column-oriented format
 
 setAs("dgCMatrix", "dsCMatrix",
-      function(from) {
-	  if(!exists(".warn.dsC")) { ## now only warn *once* ..
-	      warning("as(.,\"dsCMatrix\") is deprecated; do use as(., \"symmetricMatrix\")")
-	      assign(".warn.dsC", "DONE", envir = .GlobalEnv)
-	  }
+      function(from) { ## r2130 ... | 2008-03-14 | added deprecation warning
+	  warning("as(.,\"dsCMatrix\") is deprecated (since 2008); do use as(., \"symmetricMatrix\")")
 	  as(from, "symmetricMatrix")
       })
 
@@ -44,6 +41,20 @@ setAs("dsCMatrix", "dgCMatrix",
 setAs("dsCMatrix", "dsyMatrix",
       function(from) as(from, "denseMatrix"))
 
+##' Check if \code{name} (== "[sS][pP][dD]Cholesky") fits the values of the
+##' logicals (perm, LDL, super).
+##' @param name a string such as "sPdCholesky"
+##' @param perm also known as \code{pivot}
+##' @param LDL
+##' @param super
+##' @return logical: TRUE if the name matches
+.chkName.CHM <- function(name, perm, LDL, super)
+    .Call(R_chkName_Cholesky, name, perm, LDL, super)
+## ../src/dsCMatrix.c
+
+.CHM.factor.name <- function(perm, LDL, super)
+    .Call(R_chm_factor_name, perm, LDL, super)
+
 ## have rather tril() and triu() methods than
 ## setAs("dsCMatrix", "dtCMatrix", ....)
 setMethod("tril", "dsCMatrix",
@@ -99,8 +110,8 @@ setMethod("solve", signature(a = "dsCMatrix", b = "numeric"),
 	  solve.dsC.mat(a, .Call(dup_mMatrix_as_dgeMatrix, b)),
 	  valueClass = "dgeMatrix")
 
-## `` Fully-sparse'' solve() :
-solve.dsC.dsC <- function(a,b, tol) {
+## ``Fully-sparse'' solve() -- only used here; separate function for debugging etc
+solve.dsC.dC <- function(a,b, tol) {
     r <- tryCatch(.Call(dsCMatrix_Csparse_solve, a, b),
 		  error=function(e)NULL, warning=function(w)NULL)
     if(is.null(r)) { ## cholmod factorization was not ok
@@ -119,11 +130,11 @@ setMethod("solve", signature(a = "dsCMatrix", b = "dsparseMatrix"),
 		  cb <- getClassDef(class(b <- as(b, "CsparseMatrix")))
 	      if (extends(cb, "symmetricMatrix")) ## not supported (yet) by cholmod_spsolve
 		  b <- as(b, "dgCMatrix")
-	      solve.dsC.dsC(a,b)
+	      solve.dsC.dC(a,b)
 	  })
 
 setMethod("solve", signature(a = "dsCMatrix", b = "missing"),
-	  function(a, b, ...) solve(a, .trDiagonal(nrow(a), unitri=FALSE)))
+	  function(a, b, ...) solve(a, .trDiagonal(nrow(a), unitri=FALSE), ...))
 
 
 
