@@ -3,10 +3,9 @@
 /* ========================================================================= */
 
 /* ------------------------------------------------------------------------- */
-/* AMD Version 2.0, Copyright (c) 2006 by Timothy A. Davis,		     */
+/* AMD, Copyright (c) Timothy A. Davis,					     */
 /* Patrick R. Amestoy, and Iain S. Duff.  See ../README.txt for License.     */
-/* email: davis at cise.ufl.edu    CISE Department, Univ. of Florida.        */
-/* web: http://www.cise.ufl.edu/research/sparse/amd                          */
+/* email: DrTimothyAldenDavis@gmail.com                                      */
 /* ------------------------------------------------------------------------- */
 
 #include <stdlib.h>
@@ -77,6 +76,8 @@ void *(*amd_calloc) (size_t, size_t) = NULL ;
 int (*amd_printf) (const char *, ...) = mexPrintf ;
 #else
 #include <stdio.h>
+#include <R_ext/Print.h>
+#define printf Rprintf
 int (*amd_printf) (const char *, ...) = printf ;
 #endif
 #else
