@@ -17,13 +17,22 @@ SEXP Csparse_diagN2U(SEXP x);
 SEXP Csparse_drop(SEXP x, SEXP tol);
 SEXP Csparse_horzcat(SEXP x, SEXP y);
 SEXP Csparse_submatrix(SEXP x, SEXP i, SEXP j);
+SEXP dCsparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value);
+SEXP lCsparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value);
+SEXP iCsparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value);
+SEXP nCsparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value);
+SEXP zCsparse_subassign(SEXP x, SEXP i_, SEXP j_, SEXP value);
 SEXP Csparse_symmetric_to_general(SEXP x);
 SEXP Csparse_general_to_symmetric(SEXP x, SEXP uplo);
 SEXP Csparse_MatrixMarket(SEXP x, SEXP fname);
+SEXP Csparse_sort (SEXP x);
 SEXP Csparse_to_Tsparse(SEXP x, SEXP tri);
 SEXP Csparse_to_dense(SEXP x);
 SEXP Csparse_to_nz_pattern(SEXP x, SEXP tri);
+SEXP nz_pattern_to_Csparse(SEXP x, SEXP res_kind);
+SEXP nz2Csparse           (SEXP x, enum x_slot_kind r_kind);
 SEXP Csparse_to_matrix(SEXP x);
+SEXP Csparse_to_vector(SEXP x);
 SEXP Csparse_transpose(SEXP x, SEXP tri);
 SEXP Csparse_validate (SEXP x);
 SEXP Csparse_validate2(SEXP x, SEXP maybe_modify);
@@ -35,6 +44,7 @@ SEXP Rsparse_validate(SEXP x);
 SEXP diag_tC_ptr(int n, int *x_p, double *x_x, int *perm, SEXP resultKind);
 SEXP diag_tC(SEXP pslot, SEXP xslot, SEXP perm_slot, SEXP resultKind);
 
+// FIXME: these are nowhere used (are they?)
 SEXP create_Csparse(char* cls, int* i, int* j, int* p, int np,
 		    void* x, int nnz, int* dims, SEXP dimnames,
 		    int index1);
